#! /usr/bin/env python3

import sys
import argparse
from pathlib import Path
from .clean import clean
from .converttomarkdown import convert_to_markdown

import requests
from bs4 import BeautifulSoup


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--url', type=str)
    parser.add_argument('--file', type=str)
    parser.add_argument('--cache', action="store_true")

    return parser.parse_args()

def get_html_string(file: str or Path = None, url: str = None, cache: bool = False):
    """Returns a string, hopefully of <html></html>. Will return in order of precedence:
    `file`'s contents, `the response.text of a response object if `url` is provided, or `stdin`
    """
    if file:
        return Path(file).read_text()
    if url:
        r = requests.get(url)
        if cache:
            Path("r-html.html").write_text(r.text)
        return r.text
    return "".join(sys.stdin.readlines())

def load(url, cache=False):
    html_string = get_html_string(url=url, cache=cache)
    clean_and_convert_to_markdown(html_string)

def clean_and_convert_to_markdown(html_string: str):
    html = BeautifulSoup(html_string, features="html.parser")
    try:
        body = clean(html.body)
        print(convert_to_markdown(body))
    except TypeError:
        print("Unable to find <body> tag.")

def main():
    args = parse_args()
    html_string = get_html_string(args.file, args.url, args.cache)
    clean_and_convert_to_markdown(html_string)
    
if __name__ == "__main__":
    main()


from bs4 import Comment
from  bs4.element import Tag
from typing import Iterable


def clean(body: Tag) -> Tag:
    """Clean an HTML element by:
    1. Removing <head>, <script>, <style> and <noscript> elements and their contents.
    2. Replacing <span> elements with their contents. 
    3. Removing all attributes except 'src', 'alt', 'href' and 'srcset' from all elements except <a> and <img>."""
    
    TAGS_THAT_CAN_KEEP_THEIR_ATTRS = ["a", "img"]
    ATTRIBUTES_ALLOWED_TO_KEEP = ["src", "alt", "href", "srcset"]
    BLOCKED_TAGS = ["style", "script", "noscript", "head"]
    REPLACED_TAGS = ["span"]

    return _clean(body, 
                 blocked_tags=BLOCKED_TAGS, 
                 replaced_tags=REPLACED_TAGS, 
                 tags_that_can_keep_their_attrs =TAGS_THAT_CAN_KEEP_THEIR_ATTRS,
                 allowed_attrs = ATTRIBUTES_ALLOWED_TO_KEEP)


def _clean(tag: Tag, *, blocked_tags: Iterable[str], replaced_tags: Iterable[str], tags_that_can_keep_their_attrs: Iterable[str], allowed_attrs: Iterable[str]) -> Tag:
    for element in tag(text=lambda text: isinstance(text, Comment)):
        element.extract()
    for t in tag.find_all(blocked_tags):
        t.decompose()
    for t in tag.find_all(replaced_tags):
        t.unwrap()

    for t in tag.find_all():
        if t.name not in tags_that_can_keep_their_attrs:
            t.attrs.clear()
        else:
            t.attrs = {k:v for k, v in t.attrs.items() if k in allowed_attrs}
    return tag


import re

from markdownify import markdownify as md
from bs4.element import Tag


def convert_to_markdown(cleaned_body: Tag) -> str:
    body_str = str(cleaned_body)
    markdown = md(body_str)
    
    # Keep at most 2 adjacent newlines
    markdown = markdown.replace("\r\n","\n")
    markdown = re.sub("\n{2,}", "\n\n", markdown)
    
    return markdown